/*
	MarkovGen - A Markov Chain Program
	Copyright (C) 2012-2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef MARKOVCHAIN_HPP
#define MARKOVCHAIN_HPP

#include <cstdint>
#include <list>
#include <map>
#include <random>
#include <vector>

class MarkovChain
{
	struct MapTarget
	{
		/* Note: std::map stores elements in order */
		std::map<std::vector<std::string>, uint64_t> weights;
		uint64_t weight_sum;
	};
	
	public:
		virtual ~MarkovChain() = default;
		MarkovChain(uint32_t max_match, uint32_t max_gen);
		
		void ResetHistory();
		void Record(const std::string &c);
		std::vector<std::string> Generate(uint32_t min_match, uint32_t max_match, uint32_t min_gen, uint32_t max_gen);
		std::vector<std::string> GenerateMatchGen(uint32_t match_length, uint32_t gen_length);
		
	private:
		void RecordMatchGen(const std::vector<std::string> &match, const std::vector<std::string> &gen);
		void AddToHistory(const std::string &c);
		
		std::mt19937_64 m_random;
		/*
		 * Key: list of strings to match sequentially
		 * Value: Map from generation size to map model
		 */
		std::map<std::vector<std::string>, std::map<uint32_t, MapTarget>> m_weights;
		std::list<std::string> m_history;
		uint32_t m_max_match;
		uint32_t m_max_gen;
};

#endif
