/*
	MarkovGen - A Markov Chain Program
	Copyright (C) 2012-2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef UTIL_HPP
#define UTIL_HPP

#ifndef NDEBUG
#define DEBUG
#endif

#ifdef DEBUG
#define ERROR_POS_DEFAULT true
#else
#define ERROR_POS_DEFAULT false
#endif

#include <cstdint>
#include <string>

#define FormatError(...) Error(__FILE__, __LINE__, __VA_ARGS__)
class Error
{
	public:
		virtual ~Error() = default;
		Error(const char *file, int line, const char *format, ...)
			#ifdef __GNUC__
			__attribute__((format(printf, 4, 5)))
			#endif
		;
		
		void OutputMessage(bool pos = ERROR_POS_DEFAULT, FILE *f = stderr) const;
		
	private:
		std::string m_message;
		const char *m_file;
		int m_line;
};

class File
{
	struct FD;
	public:
		class EndOfFile{};
		
		virtual ~File();
		File(int fd, bool own = true);
		File(FILE *f, bool own = true);
		File(std::string filename, bool write = false);
		File(const File &file);
		File &operator=(const File &file);
		
		void Read(void *buffer, size_t length);
		void Write(void *buffer, size_t length);
		
		uint64_t ReadInt(unsigned int n);
		void Skip(uint64_t count);
		
	private:
		FD *m_fd;
};

class Uncopyable
{
	public:
		Uncopyable() = default;
		
	private:
		Uncopyable(const Uncopyable &) = delete;
		Uncopyable &operator=(const Uncopyable &) = delete;
};

#ifdef __GNUC__
	std::string FormatString(const char *format, ...)  __attribute__((format(printf, 1, 2)));
#else
	std::string FormatString(const char *format, ...);
#endif

#endif
